package com.ruoyi.service.impl;

import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.ruoyi.domain.ReviewDetails;
import com.ruoyi.domain.ReviewKeyPoint;
import com.ruoyi.mapper.ReviewDetailsMapper;
import com.ruoyi.mapper.ReviewStandardMapper;
import com.ruoyi.service.ReviewDetailsService;
import com.ruoyi.service.ReviewStandardService;
import com.ruoyi.web.request.DeleteDetailsRequest;
import com.ruoyi.web.response.*;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

@Service
public class ReviewDetailsServiceImpl extends ServiceImpl<ReviewDetailsMapper, ReviewDetails> implements ReviewDetailsService {

    @Autowired
    private ReviewDetailsMapper reviewDetailsMapper;

    @Autowired
    private ReviewStandardMapper reviewStandardMapper;

    @Override
    public List<ReviewDetails> findByKeyPointsList(List<ReviewKeyPoint> reviewKeyPointList) {
        return reviewDetailsMapper.findByKeyPointList(reviewKeyPointList);
    }

    /*
     * 删除细则按钮
     * */
    @Override
    public List<DeleteDetailsResponse> getDeleteDetailsResponseList(DeleteDetailsRequest deleteDetailsRequest) {
        // 获取所有要点ID列表
        List<Long> idList = reviewStandardMapper.getDelKeypointIdList(deleteDetailsRequest);

        // 获取所有的细则列表
        List<KeypointList> keypointLists = reviewStandardMapper.getKeypointListByIdList(idList);

        // 将要点列表按reviewStandardId分组
        Map<Long, List<KeypointList>> groupKeypoint = keypointLists.stream()
                .collect(Collectors.groupingBy(KeypointList::getReviewStandardId));

        // 创建返回结果列表
        List<DeleteDetailsResponse> deleteDetailsResponseList = new ArrayList<>();
        for (Map.Entry<Long, List<KeypointList>> entry : groupKeypoint.entrySet()) {
            // 获取标准信息
            ReviewStandardResponseNew reviewStandardResponseNew = reviewStandardMapper.selectReviewStandardById(entry.getKey());

            // 处理标准信息为null的情况
            if (reviewStandardResponseNew == null) {
                continue;
            }

            // 组装 DeleteDetailsResponse
            DeleteDetailsResponse deleteDetailsResponse = new DeleteDetailsResponse();
            deleteDetailsResponse.setId(reviewStandardResponseNew.getId());
            deleteDetailsResponse.setChapter(reviewStandardResponseNew.getChapter());
            deleteDetailsResponse.setText(reviewStandardResponseNew.getText());
            deleteDetailsResponse.setStandardId(reviewStandardResponseNew.getStandardId());
            deleteDetailsResponse.setStandardNo(reviewStandardResponseNew.getStandardNo());

            // 组装 DeleteDetailsList
            List<DeleteDetailsList> deleteDetailsLists = new ArrayList<>();
            for (KeypointList keypoint : entry.getValue()) {
                DeleteDetailsList deleteDetailsList = new DeleteDetailsList();
                deleteDetailsList.setId(keypoint.getId());
                deleteDetailsList.setText(keypoint.getText());
                deleteDetailsList.setReviewStandardId(keypoint.getReviewStandardId());

                // 获取与 KeypointList 关联的 DeleteDetails
                List<DeleteDetails> deleteDetails = reviewStandardMapper.getDeleteDetailsByKeypointId(keypoint.getId());
                deleteDetailsList.setDeleteDetailsList(deleteDetails);

                deleteDetailsLists.add(deleteDetailsList);
            }

            deleteDetailsResponse.setDeleteDetailsLists(deleteDetailsLists);

            // 添加到返回列表
            deleteDetailsResponseList.add(deleteDetailsResponse);
        }

        return deleteDetailsResponseList;
    }
}
