package com.ruoyi.service.impl;

import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.ruoyi.mapper.ReviewStandardMapper;
import com.ruoyi.mapper.ReviewUsecaseMapper;
import com.ruoyi.service.ReviewUsecaseService;
import com.ruoyi.web.request.GetUseCaseRequest;
import com.ruoyi.web.request.ReviewKeyPointListRequest;
import com.ruoyi.web.response.GetUseCaseResponse;
import com.ruoyi.web.response.ReviewStandardResponse;
import com.ruoyi.web.response.ReviewStandardResponseNew;
import com.ruoyi.web.response.UseCaseResponseDTO;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

@Service
public class ReviewUsecaseServiceImpl implements ReviewUsecaseService {
    @Autowired
    private ReviewUsecaseMapper reviewUsecaseMapper;

    @Autowired
    private ReviewStandardMapper reviewStandardMapper;

    /*
     * 查询所有测试用例
     * */
    @Override
    public List<GetUseCaseResponse> getUseCaseListByIds(List<Long> idList) {
        if (idList.size() == 0){
            return new ArrayList<>();
        }
        //获取所有测试用例
        List<UseCaseResponseDTO> useCaseResponseDTOList = reviewUsecaseMapper.getUseCaseListByIdList(idList);
        // 按 reviewStandardId 分组
        Map<Long, List<UseCaseResponseDTO>> groupedUseCases = useCaseResponseDTOList.stream()
                .collect(Collectors.groupingBy(UseCaseResponseDTO::getReviewStandardId));
        // 创建返回结果列表
        List<GetUseCaseResponse> getUseCaseResponseList = new ArrayList<>();
        for (Map.Entry<Long, List<UseCaseResponseDTO>> entry : groupedUseCases.entrySet()) {
            // 获取标准信息
            ReviewStandardResponseNew reviewStandardResponse = reviewStandardMapper.selectReviewStandardById(entry.getKey());

            // 处理标准信息为 null 的情况
            if (reviewStandardResponse == null) {
                // 如果标准信息为 null，则跳过该条记录，或者根据需求处理
                continue;
            }

            // 组装 GetUseCaseResponse 对象
            GetUseCaseResponse getUseCaseResponse = new GetUseCaseResponse();
            getUseCaseResponse.setId(reviewStandardResponse.getId());
            getUseCaseResponse.setText(reviewStandardResponse.getText());
            getUseCaseResponse.setStandardId(reviewStandardResponse.getStandardId());
            getUseCaseResponse.setStandardNo(reviewStandardResponse.getStandardNo());
            getUseCaseResponse.setUseCaseResponseDTOList(entry.getValue());

            // 添加到返回列表
            getUseCaseResponseList.add(getUseCaseResponse);
        }
        return getUseCaseResponseList;
    }
}
