package com.ruoyi.service.impl;

import cn.hutool.http.HttpUtil;
import com.alibaba.fastjson2.JSONObject;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.ruoyi.common.utils.StringUtils;
import com.ruoyi.domain.TestRecords;
import com.ruoyi.domain.vo.AttachmentVO;
import com.ruoyi.domain.vo.CaseResultVO;
import com.ruoyi.domain.vo.StepResultVO;
import com.ruoyi.mapper.TestRecordsMapper;
import com.ruoyi.service.TestRecordsService;
import org.commonmark.node.Node;
import org.commonmark.parser.Parser;
import org.commonmark.renderer.html.HtmlRenderer;
import org.jsoup.Jsoup;
import org.jsoup.nodes.Entities;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.text.SimpleDateFormat;
import java.util.*;


@Transactional
@Service
public class TestRecordsServiceImpl extends ServiceImpl<TestRecordsMapper, TestRecords> implements TestRecordsService {

    @Autowired
    private TestRecordsService testRecordsService;

    @Autowired
    private TestRecordsMapper testRecordsMapper;


    @Override
    public void sendPost(String id,Long taskId) {

        Map<String, Object> map = new HashMap<>();
        map.put("id", id);
        map.put("verbose", "ALL");

        //以post形式请求接口
        String result= HttpUtil.post("http://10.12.48.77:8090/DescribeProjectTestResult",JSONObject.toJSONString(map));

        JSONObject jsonObject = JSONObject.parseObject(result);

        // 获取项目id
        String projectId = (String) jsonObject.get("id");

        // 获取项目用例结果列表
        List<CaseResultVO> caseResultVOS = jsonObject.getList("case_result_list", CaseResultVO.class);

        // 获取本地存储的列表
        List<TestRecords> recordsList = testRecordsService.findByTaskId(id);

        List<TestRecords> list = new ArrayList<>();
        if (caseResultVOS != null && caseResultVOS.size() != 0) {
            for (CaseResultVO caseResultVO : caseResultVOS) {
                // 检查 usecaseId 是否在 recordsList 中存在
                boolean exists = false;

                if (recordsList != null && recordsList.size() != 0) {

                    for (TestRecords records : recordsList) {
                        if (Objects.equals(caseResultVO.getId(), records.getUsecaseId())) {
                            exists = true;

                            if (!Objects.equals(caseResultVO.getStatus(), records.getTestResult())) {
                                records.setTestResult(caseResultVO.getStatus());
                                records.setRemediation(caseResultVO.getRemediation());
                                testRecordsService.updateById(records);
                            }
                            break;
                        }
                    }
                }

                if ((Objects.equals(caseResultVO.getStatus(), "PASSED") || Objects.equals(caseResultVO.getStatus(), "FAILED")) && !exists) {
                    TestRecords testRecords = new TestRecords();
                    testRecords.setProjectId(projectId);
                    testRecords.setTaskId(taskId);
                    testRecords.setUsecase(caseResultVO.getName());
                    testRecords.setUsecaseId(caseResultVO.getId());
                    testRecords.setDescription(caseResultVO.getDescription());
                    testRecords.setRiskLevel(caseResultVO.getRisk_level() + "_risk");
                    testRecords.setTestResult(caseResultVO.getStatus());
                    testRecords.setRemediation(caseResultVO.getRemediation());
                    testRecords.setUsecaseNo(caseResultVO.getSequence());
                    //测试开始时间
                    if (caseResultVO.getStart_time() != null) {
                        Date date = new Date(caseResultVO.getStart_time());
                        SimpleDateFormat sdf = new SimpleDateFormat("yyyy年MM月dd日");
                        String testTime = sdf.format(date);
                        testRecords.setTestTime(testTime);
                    }

                    // 用例步骤
                    List<StepResultVO> stepResultVOS = caseResultVO.getStep_result_list();

                    if (stepResultVOS != null && stepResultVOS.size() != 0) {
                        List<String> stepList = new ArrayList<>();
                        List<String> testDetailsList = new ArrayList<>();
                        int index = 1; // 序号从1开始
                        StringBuilder testDetailsVO = new StringBuilder();
                        for (StepResultVO stepResultVO :stepResultVOS) {
                            // 测试方法
                            String stepName = stepResultVO.getName();

                            // 给每个元素加上序号
                            String stepWithIndex = index + "." + stepName;

                            stepList.add(stepWithIndex);
                            index++;

                            // 测试详情
                            String testDetails = stepResultVO.getDescription();
                            if (testDetails != null) {
                                testDetails = convertMarkdownToHtml(testDetails);
                            }
                            testDetailsVO.append(testDetails).append("<br> ");

                            // 步骤附件列表
                            List<AttachmentVO> attachmentVOS = stepResultVO.getAttachment_list();

                            if (attachmentVOS != null && attachmentVOS.size() != 0) {
                                // 附件id
                                StringBuilder attachmentId = new StringBuilder();
                                // 附件名称
                                StringBuilder attachmentName = new StringBuilder();
                                for (AttachmentVO vo : attachmentVOS) {
                                    attachmentId.append(vo.getId()).append("、");
                                    attachmentName.append(vo.getName()).append("、");
                                }
                                attachmentId = new StringBuilder(attachmentId.substring(0, attachmentId.length() - 1));
                                attachmentName = new StringBuilder(attachmentName.substring(0, attachmentName.length() - 1));

                                testRecords.setAttachmentId(attachmentId.toString());
                                testRecords.setAttachmentName(attachmentName.toString());
                            }
                        }
                        testDetailsVO = new StringBuilder(testDetailsVO.substring(0, testDetailsVO.length() - 1));
                        testRecords.setTestMethod(StringUtils.join(stepList,"\n"));
                        testRecords.setTestDetails(testDetailsVO.toString());
                    }

                    list.add(testRecords);

                }

            }

            testRecordsService.saveBatch(list);
        }
    }

    @Override
    public void singleSendPost(String id, Long taskId) {
        Map<String, Object> map = new HashMap<>();
        map.put("id", id);
        map.put("verbose", "ALL");

        //以post形式请求接口
        String result= HttpUtil.post("http://10.12.48.77:8090/DescribeProjectTestResult",JSONObject.toJSONString(map));

        JSONObject jsonObject = JSONObject.parseObject(result);

        // 获取项目id
        String projectId = (String) jsonObject.get("id");

        // 获取项目用例结果列表
        List<CaseResultVO> caseResultVOS = jsonObject.getList("case_result_list", CaseResultVO.class);

        // 获取当前任务本地存储的列表
        List<TestRecords> recordsList = testRecordsService.findByTaskId(id);
        if (recordsList != null && recordsList.size() != 0) {
            testRecordsService.removeBatchByIds(recordsList);
        }

        List<TestRecords> list = new ArrayList<>();
        if (caseResultVOS != null && caseResultVOS.size() != 0) {
            for (CaseResultVO caseResultVO : caseResultVOS) {

                if (Objects.equals(caseResultVO.getStatus(), "PASSED") || Objects.equals(caseResultVO.getStatus(), "FAILED")) {
                    TestRecords testRecords = new TestRecords();
                    testRecords.setProjectId(projectId);
                    testRecords.setTaskId(taskId);
                    testRecords.setUsecase(caseResultVO.getName());
                    testRecords.setUsecaseId(caseResultVO.getId());
                    testRecords.setDescription(caseResultVO.getDescription());
                    testRecords.setRiskLevel(caseResultVO.getRisk_level() + "_risk");
                    testRecords.setTestResult(caseResultVO.getStatus());
                    testRecords.setRemediation(caseResultVO.getRemediation());
                    testRecords.setUsecaseNo(caseResultVO.getSequence());
                    //测试开始时间
                    if (caseResultVO.getStart_time() != null) {
                        Date date = new Date(caseResultVO.getStart_time());
                        SimpleDateFormat sdf = new SimpleDateFormat("yyyy年MM月dd日");
                        String testTime = sdf.format(date);
                        testRecords.setTestTime(testTime);
                    }

                    // 用例步骤
                    List<StepResultVO> stepResultVOS = caseResultVO.getStep_result_list();

                    if (stepResultVOS != null && stepResultVOS.size() != 0) {
                        List<String> stepList = new ArrayList<>();
                        List<String> testDetailsList = new ArrayList<>();
                        int index = 1; // 序号从1开始
                        StringBuilder testDetailsVO = new StringBuilder();
                        for (StepResultVO stepResultVO :stepResultVOS) {
                            // 测试方法
                            String stepName = stepResultVO.getName();

                            // 给每个元素加上序号
                            String stepWithIndex = index + "." + stepName;

                            stepList.add(stepWithIndex);
                            index++;

                            // 测试详情
                            String testDetails = stepResultVO.getDescription();
                            if (testDetails != null) {
                                testDetails = convertMarkdownToHtml(testDetails);
                            }
                            testDetailsVO.append(testDetails).append("<br> ");

                            // 步骤附件列表
                            List<AttachmentVO> attachmentVOS = stepResultVO.getAttachment_list();

                            if (attachmentVOS != null && attachmentVOS.size() != 0) {
                                // 附件id
                                StringBuilder attachmentId = new StringBuilder();
                                // 附件名称
                                StringBuilder attachmentName = new StringBuilder();
                                for (AttachmentVO vo : attachmentVOS) {
                                    attachmentId.append(vo.getId()).append("、");
                                    attachmentName.append(vo.getName()).append("、");
                                }
                                attachmentId = new StringBuilder(attachmentId.substring(0, attachmentId.length() - 1));
                                attachmentName = new StringBuilder(attachmentName.substring(0, attachmentName.length() - 1));

                                testRecords.setAttachmentId(attachmentId.toString());
                                testRecords.setAttachmentName(attachmentName.toString());
                            }
                        }
                        testDetailsVO = new StringBuilder(testDetailsVO.substring(0, testDetailsVO.length() - 1));
                        testRecords.setTestMethod(StringUtils.join(stepList,"\n"));
                        testRecords.setTestDetails(testDetailsVO.toString());
                    }

                    list.add(testRecords);

                }

            }

            testRecordsService.saveBatch(list);
        }
    }

    /**
     * 根据任务id获取用例
     * @param taskId
     * @return
     */
    @Override
    public List<TestRecords> selectListByTaskId(Long taskId) {
        return testRecordsMapper.selectListByTaskId(taskId);
    }

    public static String convertMarkdownToHtml(String markdownContent) {
        // 创建 Markdown 解析器
        Parser parser = Parser.builder().build();

        // 解析 Markdown
        Node document = parser.parse(markdownContent);

        // 创建 HTML 渲染器
        HtmlRenderer renderer = HtmlRenderer.builder().build();

        // 渲染 HTML
        org.jsoup.nodes.Document doc = Jsoup.parse(renderer.render(document).replaceAll("\\\\n", "<br>"));

        // jsoup标准化标签，生成闭合标签
        doc.outputSettings().syntax(org.jsoup.nodes.Document.OutputSettings.Syntax.xml);
        doc.outputSettings().escapeMode(Entities.EscapeMode.xhtml);
        return doc.body().html();
    }

    @Override
    public List<TestRecords> findByTaskId(String id) {
        return testRecordsMapper.findByTaskId(id);
    }
}


